<?php

/**
 * Class EkomiIntegrationAdminOrderActionExtender.
 *
 * Handles the ekomiIntegration Module functionalities.
 *
 * @category   System
 *
 * @author     eKomi Ltd <support@ekomi.de>
 */
class EkomiIntegrationManager
{
    /**
     * Product identifiers.
     */
    const PRODUCT_IDENTIFIER_ID = 'id';
    const PRODUCT_IDENTIFIER_SKU = 'sku';

    /**
     * The URL where the order data is sent.
     */
    const URL_TO_SEND_DATA = 'https://plugins-dashboard.ekomiapps.de/api/v1/order';

    /**
     * The Url to validate shop.
     */
    const URL_GET_SETTINGS = 'https://api.ekomi.de/v3/getSettings';

    /**
     * The Smart Widgets URL to get widgets.
     */
    const URL_SMART_WIDGETS_LOGIN = 'https://widgets.ekomi.com/api/1.0/security/login';
    const URL_SMART_WIDGETS = 'https://widgets.ekomi.com/api/1.0/product/widget/token';

    /**
     * Get Settings api output.
     */
    const GET_SETTING_API_RESPONSE = 'Access denied';

    /**
     * Enable/Disable value.
     */
    const ENABLE_VALUE = '1';
    const DISABLE_VALUE = '0';

    public $shopId = 0;
    public $shopPassword = '';

    /**
     * Initializes the class properties.
     *
     * @param int    $shopId       The shop id
     * @param string $shopPassword The shop password
     */
    public function __construct($shopId, $shopPassword)
    {
        $this->shopId = $shopId;
        $this->shopPassword = $shopPassword;
    }

    /**
     * Verifies eKomi shop id and password.
     *
     * @return bool
     */
    public function verifyAccount()
    {
        $apiUrl = self::URL_GET_SETTINGS;
        $apiUrl .= "?auth={$this->shopId}|{$this->shopPassword}";
        $apiUrl .= '&version=cust-1.0.0&type=request&charset=iso&app=PD-Gambio-30';
        $serverOutput = $this->doCurl($apiUrl, 'GET');
        if (self::GET_SETTING_API_RESPONSE == $serverOutput) {
            return false;
        }

        return true;
    }

    /**
     * Makes a curl request.
     *
     * @param string $requestUrl
     * @param string $requestType
     * @param array  $httpHeader
     * @param string $postFields
     *
     * @return mixed|string
     */
    public function doCurl($requestUrl, $requestType, $httpHeader = array(), $postFields = '')
    {
        try {
            if ((bool)gm_get_conf('EKOMIINTEGRATION_TERMS_AND_CONDITIONS') == false ) {
                throw new Exception('Ekomi terms and conditions not accepted');
            }

            $ch = curl_init();
            curl_setopt($ch, CURLOPT_URL, $requestUrl);
            curl_setopt($ch, CURLOPT_HEADER, false);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
            curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $requestType);
            if (!empty($httpHeader)) {
                curl_setopt($ch, CURLOPT_HTTPHEADER, $httpHeader);
            }

            if (!empty($postFields)) {
                curl_setopt($ch, CURLOPT_POSTFIELDS, $postFields);
            }

            $response = curl_exec($ch);
            curl_close($ch);

            return $response;
        } catch (Exception $exception) {
            return $exception->getMessage();
        }
    }

    /**
     * Gets smart widget token.
     *
     * @return string
     */
    public function getDefaultWidgetToken()
    {
        $postFields = json_encode(array('username' => $this->shopId, 'password' => $this->shopPassword));
        $httpHeader = array('Content-Type: application/json', 'Content-Length: ' . strlen($postFields));
        $response = $this->doCurl(self::URL_SMART_WIDGETS_LOGIN, 'POST', $httpHeader, $postFields);
        if (!empty($response)) {
            $responseData = json_decode($response);
            $loginToken = $responseData->token;

            return $this->_getSmartWidget($loginToken);
        }

        return '';
    }

    /**
     * Gets default smart widget token.
     *
     * @param string $loginToken
     *
     * @return string
     */
    protected function _getSmartWidget($loginToken)
    {
        $httpHeader = array('Authorization: ekomi ' . $loginToken);
        $response = $this->doCurl(self::URL_SMART_WIDGETS . '?get_default=true', 'GET', $httpHeader);
        $widgets = json_decode($response);
        if (isset($widgets[0])) {
            return $widgets[0]->token;
        }

        return '';
    }

    /**
     * Exports the order data to ekomi.
     *
     * @param int    $orderId
     * @param string $orderStatus
     */
    public function exportOrder($orderId, $orderStatus = '')
    {
        $ekomiIntegrationOrder = MainFactory::create_object('EkomiIntegrationOrder', array($orderId));
        if (!empty($orderStatus)) {
            $ekomiIntegrationOrder->setOrderStatus($orderStatus);
        }

        $ekomiIntegrationOrderStatuses = explode(',', gm_get_conf('EKOMIINTEGRATION_ORDER_STATUSES'));
        if (in_array($ekomiIntegrationOrder->getOrderStatus(), $ekomiIntegrationOrderStatuses)) {
            $orderData = $ekomiIntegrationOrder->getOrderData($this->shopId, $this->shopPassword);
            $this->sendOrderData($orderData);
        }
    }

    /**
     * Sends order data to eKomi plugins-dashboard.
     *
     * @param array $orderData
     *
     * @return string
     */
    public function sendOrderData($orderData)
    {
        if (!empty($orderData)) {
            $boundary = md5(time());
            $httpHeader = array('ContentType:multipart/form-data;boundary=' . $boundary);
            $response = $this->doCurl(
                self::URL_TO_SEND_DATA,
                'PUT',
                $httpHeader,
                json_encode($orderData)
            );

            return $response;
        }
    }

    /**
     * Gets the shop id.
     *
     * @return int The shop id
     */
    public function get_shopId()
    {
        return $this->shopId;
    }

    /**
     * Gets the shop password.
     *
     * @return string The shop Api Key
     */
    public function get_shopPassword()
    {
        return $this->shopPassword;
    }
}
